/*
 * mocktcp
 *
 * This library provides a reliable, ordered, stream-based, message 
 * transport designed to look very similar to TCP, although slightly
 * cleaner. Underneath, mocktcp uses rmessage reliable datagrams for
 * packet delivery. Rmessages uses UDP.
 *
 * -Dave Lowell
 */

#ifndef MOCKTCP_H
#define MOCKTCP_H

#include "rmessage.h"

#define MT_MAX_MTU RM_MAX_MTU

typedef struct {
	char	buf[MT_MAX_MTU];	/* buffer */
	int	bytes;			/* number of bytes in buffer */
	int	index;			/* index of next avail bytes */
} conn_buffer;

extern void mt_init(void);
extern int mt_socket(void);
extern int mt_bind(int, int);
extern int mt_listen(int);
extern int mt_accept(int, struct sockaddr_in*);
extern int mt_connect(int, char*, int);
extern int mt_send(int, void*, int, int);
extern int mt_recv(int, void*, int, int);
extern int mt_write(int, void*, int);
extern int mt_read(int, void*, int);
extern int mt_close(int);
extern int mt_bytes_ready(int);

#endif

