/*
 * Rmessage library. By Dave Lowell.
 */

#ifndef RMESSAGE_H
#define RMESSAGE_H

/**
 ** Includes
 **/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <signal.h>
#include <netdb.h>
#include <limits.h>


/**
 ** Defines
 **/

#define TRUE 1
#define FALSE 0

#define RM_MAX_DESCRIPTORS 	1024
#define RM_MAX_ID 		UINT_MAX
#define RM_WINDOW 		(RM_MAX_ID/2)
#define RM_REG 			1
#define RM_ACK 			2
#define RM_EOF 			3
#define RM_TIMER_SIG		SIGSYS
#define RM_MAX_MTU 		40000

/* Tunable constants */

#define RM_TIMER_PERIOD		50 		/* period in ms */
#define RM_SIGS_PER_RETRANS	12		/* retrans every N timer ints */

#define RM_MAX_OUTSTANDING 	300		/* max outstanding msgs/sock */
#define RM_MAX_OUT_BYTES 	(1024*1024)	/* max outstanding bytes/sock */

#ifdef UNDEF

/* These four values are decent for inter and intra node IPC */
#define RM_TIMER_PERIOD		20 		/* period in ms */
#define RM_SIGS_PER_RETRANS	8		/* retrans every N timer ints */

#define RM_MAX_OUTSTANDING 	300		/* max outstanding msgs/sock */
#define RM_MAX_OUT_BYTES 	(2024*1024)	/* max outstanding bytes/sock */

/* These four values work well for inter-node IPC on switched 100-baseT */
#define RM_TIMER_PERIOD		50 		/* period in ms */
#define RM_SIGS_PER_RETRANS	12		/* retrans every N timer ints */

#define RM_MAX_OUTSTANDING 	300		/* max outstanding msgs/sock */
#define RM_MAX_OUT_BYTES 	(2048*1024)	/* max outstanding bytes/sock */

/* These four values work well for local IPC */
#define RM_TIMER_PERIOD		30 		/* period in ms */
#define RM_SIGS_PER_RETRANS	2		/* retrans every N timer ints */

#define RM_MAX_OUTSTANDING 	200		/* max outstanding msgs/sock */
#define RM_MAX_OUT_BYTES 	(1024*1024)	/* max outstanding bytes/sock */

#define RM_TIMER_PERIOD		50 		/* period in ms */
#define RM_SIGS_PER_RETRANS	4		/* retrans every 4 timer ints */

#define RM_MAX_OUTSTANDING 	200		/* max outstanding msgs/sock */
#define RM_MAX_OUT_BYTES 	(1024*1024)	/* max outstanding bytes/sock */

#endif


/**
 ** Types
 **/

/* rm message header */
typedef struct {
	u_int	id;		/* mesg id. Used for ordering */
	short	type;		/* type of packet: ack or regular */
	short	len;		/* size of user data */
	u_int	rcvd;		/* ack up to mesg # 'rcvd' */
	char	data[0];	/* variable length user data */
} rm_msg;
#define RM_HEADER_SIZE (sizeof(rm_msg))

/* data per host/port */
typedef struct {
	u_int	received;	/* id max pack in seq received */
	int	do_ack;		/* send ack if true */
	int	ack_index;	/* our slot in to_ack list */
	int	s;		/* sock for sending singleton acks */
	u_long	ip;		/* dest ip for sending singleton acks */
	int	port;		/* dest port for sending singleton acks */
	int	closing;	/* flag to let us know if we've received EOF */
} connection_info;

/* message that may need to be retransmitted */
typedef struct out_msg_s {
	rm_msg	*m;		/* msg that was sent */
	int	sock;		/* socket on which it was sent */
	int	flags;		/* flags under which is was sent */
	u_long	ip;		/* host to which it was sent */
	int	port;		/* port on above host to which it was sent */
	struct out_msg_s *next;	/* next msg on list */
	struct out_msg_s *prev;	/* previous msg on list */
} outstanding_msg;


/**
 ** Public Functions
 **/

extern void rm_init(void);
extern int rm_socket(void);
extern int rm_bind(int, int);
extern int rm_connect(int, char*, int);
extern int rm_connect_ip(int, u_long, int);
extern int rm_sendto(int, void*, int, int, struct sockaddr_in*);
extern int rm_recvfrom(int, void*, int, int, struct sockaddr_in*);
extern int rm_recv(int, void*, int, int);
extern int rm_send(int, void*, int, int);
extern int rm_fill_sockaddr(struct sockaddr_in*, char*, int);
extern int rm_get_mtu(int);
extern int rm_bytes_ready(int, int);
extern void rm_drain(int);
extern int rm_close(int);
extern int rm_conn_closing(int);
extern int rm_ack(int);

#endif

